# -*- coding: utf-8 -*-

def read_terminology_entry(terminology_entry):
    terminology_items = terminology_entry.split('~')

    terminology = {}
    terminology['contextName'] = terminology_items[0]

    terminology['category'] = terminology_items[1].split("^")
    terminology['type'] = terminology_items[2].split("^")
    typeModifier = terminology_items[3].split("^")
    if any(item!='' for item in typeModifier):
        terminology['typeModifier'] = typeModifier

    anatomicRegion = terminology_items[5].split("^")
    if any(item!='' for item in anatomicRegion):
        terminology['anatomicContextName'] = terminology_items[4]
        terminology['anatomicRegion'] = anatomicRegion
        anatomicRegionModifier = terminology_items[6].split("^")
        if any(item!='' for item in anatomicRegionModifier):
            terminology['anatomicRegionModifier'] = anatomicRegionModifier

    return terminology

def read_segmentation_info(filename):
    import nrrd
    header = nrrd.read_header(filename)
    segmentation_info = {}
    segments = []
    segment_index = 0
    while True:
        prefix = "Segment{0}_".format(segment_index)
        if not prefix + "ID" in header.keys():
            break
        segment = {}
        segment["index"] = segment_index
        segment["color"] = [float(i) for i in header[prefix + "Color"].split(" ")]  # Segment0_Color:=0.501961 0.682353 0.501961
        segment["colorAutoGenerated"] = int(header[prefix + "ColorAutoGenerated"]) != 0  # Segment0_ColorAutoGenerated:=1
        segment["extent"] = [int(i) for i in header[prefix + "Extent"].split(" ")]  # Segment0_Extent:=68 203 53 211 24 118
        segment["id"] = header[prefix + "ID"]  # Segment0_ID:=Segment_1
        segment["labelValue"] = int(header[prefix + "LabelValue"])  # Segment0_LabelValue:=1
        segment["layer"] = int(header[prefix + "Layer"])  # Segment0_Layer:=0
        segment["name"] = header[prefix + "Name"]  # Segment0_Name:=Segment_1
        segment["nameAutoGenerated"] = int(header[prefix + "NameAutoGenerated"]) != 0  # Segment0_NameAutoGenerated:=1
        # Segment0_Tags:=Segmentation.Status:inprogress|TerminologyEntry:Segmentation category and type - 3D Slicer General Anatomy list
        # ~SCT^85756007^Tissue~SCT^85756007^Tissue~^^~Anatomic codes - DICOM master list~^^~^^|
        tags = {}
        tags_str = header[prefix + "Tags"].split("|")
        for tag_str in tags_str:
            tag_str = tag_str.strip()
            if not tag_str:
                continue
            key, value = tag_str.split(":", maxsplit=1)
            if key=="TerminologyEntry":
                segment["terminology"] = read_terminology_entry(value)
            elif key=="Segmentation.Status":
                segment["status"] = value
            else:
                tags[key] = value
        segment["tags"] = tags
        segments.append(segment)
        segment_index += 1
    segmentation_info["segments"] = segments
    return segmentation_info

def segment_from_name(segmentation_info, segment_name):
    for segment in segmentation_info["segments"]:
        if segment_name == segment["name"]:
            return segment
    raise KeyError("segment not found by name " + segment_name)

def segment_names(segmentation_info):
    names = []
    for segment in segmentation_info["segments"]:
        names.append(segment["name"])
    return names

def extract_segments(voxels, header, segmentation_info, segment_names_to_label_values):
    import numpy as np
    # Create empty array from last 3 dimensions (output will be flattened to a 3D array)
    output_voxels = np.zeros(voxels.shape[-3:])
    # Copy non-segmentation fields to the extracted header
    output_header = {}
    for key in header.keys():
        if not re.match("^Segment[0-9]+_.+", key):
            output_header[key] = header[key]
    # Copy extracted segments
    dims = len(voxels.shape)
    for output_segment_index, segment_name_to_label_value in enumerate(segment_names_to_label_values):
        # Copy relabeled voxel data
        segment = segment_from_name(segmentation_info, segment_name_to_label_value[0])
        input_label_value = segment["labelValue"]
        output_label_value = segment_name_to_label_value[1]
        if dims == 3:
            output_voxels[voxels == input_label_value] = output_label_value
        elif dims == 4:
            inputLayer = segment["layer"]
            output_voxels[voxels[inputLayer,:,:,:] == input_label_value] = output_label_value
        else:
            raise ValueError("Voxel array dimension is invalid")
        # Copy all segment fields corresponding to this segment
        for key in header.keys():
            prefix = "Segment{0}_".format(segment["index"])
            matched = re.match("^" + prefix + "(.+)", key)
            if not matched:
                continue
            field_name = matched.groups()[0]
            if field_name == "LabelValue":
                value = output_label_value
            elif field_name == "Layer":
                # Output is a single layer (3D volume)
                value = 0
            else:
                value = header[key]
            output_header["Segment{0}_".format(output_segment_index) + field_name] = value
    # Remove unnecessary 4th dimension (volume is collapsed into 3D)
    if dims == 4:
        # Remove "none" from "none (0,1,0) (0,0,-1) (-1.2999954223632812,0,0)"
        output_header["space directions"] = output_header["space directions"][-3:,:]
        # Remove "list" from "list domain domain domain"
        output_header["kinds"] = output_header["kinds"][-3:]
    return output_voxels, output_header



# Copyright 2008-2020 pydicom authors. See LICENSE file for details.
"""Helper functions for accessing test data.
"""

import os
from pathlib import Path

DATA_ROOT = Path(__file__).parent.joinpath('data')
"""The absolute path to the data_files directory."""

def get_testdata_file(filename):
    """Return an absolute path to the first matching dataset with filename `filename`.
    :param filename: file name (without path)
    :return: absolute path of the test data file.
    """
    absolute_path = DATA_ROOT.joinpath(filename)

    if not absolute_path.is_file():
        raise FileNotFoundError(filename)

    return os.fspath(absolute_path.resolve())



# -*- coding: utf-8 -*-

import numpy as np
import slicerio
import time
import unittest

#DATA_ROOT = os.fspath(Path(__file__).parent.joinpath('data').resolve()

class TestSegmentationRoundtrip(unittest.TestCase):
    """
    Test segmentation writing/reading roundtrip.
    """

    def setUp(self):
        pass

    def tearDown(self):
        pass

    def test_segmentation_read(self):
        """Test segmentation reading"""

        input_segmentation_filepath = slicerio.get_testdata_file('Segmentation.seg.nrrd')

        segmentation_info = slicerio.read_segmentation_info(input_segmentation_filepath)

        number_of_segments = len(segmentation_info["segments"])
        self.assertEqual(number_of_segments, 7)

        segment_names = slicerio.segment_names(segmentation_info)
        self.assertEqual(segment_names[0], 'ribs')
        self.assertEqual(segment_names[1], 'cervical vertebral column')
        self.assertEqual(segment_names[2], 'thoracic vertebral column')
        self.assertEqual(segment_names[3], 'lumbar vertebral column')
        self.assertEqual(segment_names[4], 'right lung')
        self.assertEqual(segment_names[5], 'left lung')
        self.assertEqual(segment_names[6], 'tissue')

        segment = slicerio.segment_from_name(segmentation_info, segment_names[4])
        self.assertEqual(segment['id'], 'Segment_5')
        self.assertEqual(segment['name'], 'right lung')
        self.assertEqual(segment['nameAutoGenerated'], True)
        self.assertEqual(segment['color'], [0.0862745, 0.772549, 0.278431])
        self.assertEqual(segment['colorAutoGenerated'], False)
        self.assertEqual(segment['labelValue'], 5)
        self.assertEqual(segment['layer'], 0)
        self.assertEqual(segment['extent'], [0, 124, 0, 127, 0, 33])
        self.assertEqual(segment['status'], 'inprogress')

        terminology = segment["terminology"]
        self.assertEqual(terminology['contextName'], 'Segmentation category and type - 3D Slicer General Anatomy list')
        self.assertEqual(terminology['category'], ['SCT', '123037004', 'Anatomical Structure'])
        self.assertEqual(terminology['type'], [SCT^39607008^Lung])
        self.assertEqual(terminology['typeModifier'], [SCT^24028007^Right])
        self.assertEqual('anatomicContextName' in terminology, False)
        self.assertEqual('anatomicRegion' in terminology, False)
        self.assertEqual('anatomicRegionModifier' in terminology, False)


if __name__ == '__main__':
    unittest.main()



"""slicerio package -- Python utilities for readin and writing 3D Slicer supporte data sets.

-----------
Quick Start
-----------

See the files in the examples directory that came with this package for more examples.

Any questions or need help? Post on 3D Slicer forum: https://discourse.slicer.org

Bugs and other issues can be reported in the issue tracker: https://www.github.com/lassoan/slicerio

"""

from .segmentation import extract_segments, read_segmentation_info, segment_from_name, segment_names
from .data_helper import get_testdata_file
from ._version import __version__, __version_info__

__all__ = [
   'extract_segments',
   'read_segmentation_info',
   'segment_from_name',
   'segment_names',
   '__version__',
   '__version_info__'
   ]



#!/usr/bin/env python

import os.path
from setuptools import setup, find_packages

# Get __version__ from slicerio/_version.py
base_dir = os.path.dirname(os.path.realpath(__file__))
version_file = os.path.join(base_dir, 'slicerio', '_version.py')
print("base_dir="+base_dir)
print("version_file="+version_file)
with open(version_file) as f:
    exec(f.read())
# pylint/flake does not know that this script injects a variable, so we need to disable checks
VERSION = __version__  # pylint:disable=undefined-variable  # noqa: F821

# Get long description from README.md
base_path = os.path.abspath(os.path.dirname(__file__))
with open(os.path.join(base_path, 'README.md')) as f:
    LONG_DESCRIPTION = f.read() 

opts = dict(
    name="slicerio",
    python_requires='>=3.6',
    version=VERSION,
    maintainer="Andras Lasso",
    maintainer_email="lasso@queensu.ca",
    author="Andras Lasso, Daniel Hoyer Iversen, Kyle Sunderland",
    author_email="lasso@queensu.ca",
    description="Utilities for 3D Slicer",
    long_description=LONG_DESCRIPTION,
    long_description_content_type='text/markdown',
    url="https://github.com/lassoan/slicerio",
    download_url="https://github.com/lassoan/slicerio/archive/master.zip",
    keywords="3DSlicer medical imaging segmentation",
    classifiers=[
      "License :: OSI Approved :: MIT License",
      "Intended Audience :: Developers",
      "Intended Audience :: Healthcare Industry",
      "Intended Audience :: Science/Research",
      "Development Status :: 4 - Beta",
      "Programming Language :: Python :: 3",
      "Operating System :: OS Independent",
      "Topic :: Scientific/Engineering :: Medical Science Apps.",
      "Topic :: System :: Networking"
      ],
    packages=find_packages(),
    install_requires=['nrrd', 'numpy'],
)

if __name__ == '__main__':
    setup(**opts)
